@@ -15,14 +15,24 @@
  * being a 'kpatch' prefix added to the __do_sys##name() function name.  This
  * causes kpatch-build to treat it as a new function (due to
  * its new name), and its caller __se_sys##name() function is inlined by its own
- * caller __x64_sys##name() function, which has an fentry hook.
-
+ * caller __x64_sys##name() function, which has an fentry hook.  Since the
+ * kpatch versions do not provide SYSCALL_METADATA, specifically entries in the
+ * __syscalls_metadata and _ftrace_events sections, provide dummy values in
+ * these sections and instruct kpatch-build to ignore changes to them.
+ *
  * To patch a syscall, just replace the use of the SYSCALL_DEFINE1 (or similar)
  * macro with the "KPATCH_" prefixed version.
  */
 
-#define KPATCH_IGNORE_SYSCALL_SECTIONS					\
+#define KPATCH_SYSCALL_METADATA(sname)					\
+	static struct syscall_metadata __used				\
+	  __section("__syscalls_metadata")			 	\
+	  *__p_syscall_meta_##sname = NULL;				\
 	KPATCH_IGNORE_SECTION("__syscalls_metadata");			\
+									\
+	static struct trace_event_call __used				\
+	  __section("_ftrace_events")					\
+	  *__event_enter_##sname = NULL;				\
 	KPATCH_IGNORE_SECTION("_ftrace_events")
 
 #define KPATCH_SYSCALL_DEFINE1(name, ...) KPATCH_SYSCALL_DEFINEx(1, _##name, __VA_ARGS__)
@@ -33,7 +43,7 @@
 #define KPATCH_SYSCALL_DEFINE6(name, ...) KPATCH_SYSCALL_DEFINEx(6, _##name, __VA_ARGS__)
 
 #define KPATCH_SYSCALL_DEFINEx(x, sname, ...)				\
-	KPATCH_IGNORE_SYSCALL_SECTIONS;					\
+	KPATCH_SYSCALL_METADATA(sname);					\
 	__KPATCH_SYSCALL_DEFINEx(x, sname, __VA_ARGS__)
 
 #ifdef CONFIG_X86_64
@@ -81,41 +91,25 @@
 
 #elif defined(CONFIG_S390)
 
-/* arch/s390/include/asm/syscall_wrapper.h versions */
 # if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 3, 0)
-
-#define __KPATCH_S390_SYS_STUBx(x, name, ...)                                          \
-	long __s390_sys##name(struct pt_regs *regs);                            \
-	ALLOW_ERROR_INJECTION(__s390_sys##name, ERRNO);                         \
-	static inline long ___se_sys##name(__MAP(x, __SC_LONG, __VA_ARGS__));   \
-	long __s390_sys##name(struct pt_regs *regs)                             \
-	{                                                                       \
-		return ___se_sys##name(SC_S390_REGS_TO_ARGS(x, __VA_ARGS__));   \
-	}                                                                       \
-	static inline long ___se_sys##name(__MAP(x, __SC_LONG, __VA_ARGS__))    \
-	{                                                                       \
-		__MAP(x, __SC_TEST, __VA_ARGS__);                               \
-		return __kpatch_do_sys##name(__MAP(x, __SC_COMPAT_CAST, __VA_ARGS__)); \
-	}
-
-#define __KPATCH_SYSCALL_DEFINEx(x, name, ...)					\
-       long __s390x_sys##name(struct pt_regs *regs);                           \
-       ALLOW_ERROR_INJECTION(__s390x_sys##name, ERRNO);                        \
-       static inline long __se_sys##name(__MAP(x, __SC_LONG, __VA_ARGS__));    \
-       static inline long __kpatch_do_sys##name(__MAP(x, __SC_DECL, __VA_ARGS__));    \
-       __KPATCH_S390_SYS_STUBx(x, name, __VA_ARGS__);                                 \
-       long __s390x_sys##name(struct pt_regs *regs)                            \
-       {                                                                       \
-               return __se_sys##name(SC_S390_REGS_TO_ARGS(x, __VA_ARGS__));    \
-       }                                                                       \
-       static inline long __se_sys##name(__MAP(x, __SC_LONG, __VA_ARGS__))     \
-       {                                                                       \
-               __MAP(x, __SC_TEST, __VA_ARGS__);                               \
-               return __kpatch_do_sys##name(__MAP(x, __SC_CAST, __VA_ARGS__));        \
-       }                                                                       \
-       static inline long __kpatch_do_sys##name(__MAP(x, __SC_DECL, __VA_ARGS__))
+#  define KPATCH_SYSCALL_WRAPPERS_V2
 # else
+#  define KPATCH_SYSCALL_WRAPPERS_V1
+# endif
 
+# if defined(RHEL_RELEASE_CODE)
+#  if RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(9, 6)
+#   define KPATCH_SYSCALL_WRAPPERS_V2
+#  else
+#   define KPATCH_SYSCALL_WRAPPERS_V1
+#  endif
+# endif
+
+
+/* arch/s390/include/asm/syscall_wrapper.h versions */
+#if defined(KPATCH_SYSCALL_WRAPPERS_V1)
+
+#if defined(CONFIG_COMPAT)
 #define __KPATCH_S390_SYS_STUBx(x, name, ...)					\
 	long __s390_sys##name(struct pt_regs *regs);				\
 	ALLOW_ERROR_INJECTION(__s390_sys##name, ERRNO);				\
@@ -126,6 +120,9 @@
 		__MAP(x,__SC_TEST,__VA_ARGS__);					\
 		return ret;							\
 	}
+#else
+#define __KPATCH_S390_SYS_STUBx(x, name, ...)
+#endif /* CONFIG_COMPAT */
 
 #define __KPATCH_SYSCALL_DEFINEx(x, name, ...)						\
 	__diag_push();									\
@@ -147,7 +144,44 @@
 	__diag_pop();									\
 	static inline long __kpatch_do_sys##name(__MAP(x,__SC_DECL,__VA_ARGS__))
 
-#endif /* LINUX_VERSION_CODE */
+# else /* KPATCH_SYSCALL_WRAPPERS_V2 */
+
+#if defined(CONFIG_COMPAT)
+#define __KPATCH_S390_SYS_STUBx(x, name, ...)                                          \
+	long __s390_sys##name(struct pt_regs *regs);                            \
+	ALLOW_ERROR_INJECTION(__s390_sys##name, ERRNO);                         \
+	static inline long ___se_sys##name(__MAP(x, __SC_LONG, __VA_ARGS__));   \
+	long __s390_sys##name(struct pt_regs *regs)                             \
+	{                                                                       \
+		return ___se_sys##name(SC_S390_REGS_TO_ARGS(x, __VA_ARGS__));   \
+	}                                                                       \
+	static inline long ___se_sys##name(__MAP(x, __SC_LONG, __VA_ARGS__))    \
+	{                                                                       \
+		__MAP(x, __SC_TEST, __VA_ARGS__);                               \
+		return __kpatch_do_sys##name(__MAP(x, __SC_COMPAT_CAST, __VA_ARGS__)); \
+	}
+#else
+#define __KPATCH_S390_SYS_STUBx(x, name, ...)
+#endif /* CONFIG_COMPAT */
+
+#define __KPATCH_SYSCALL_DEFINEx(x, name, ...)					\
+       long __s390x_sys##name(struct pt_regs *regs);                           \
+       ALLOW_ERROR_INJECTION(__s390x_sys##name, ERRNO);                        \
+       static inline long __se_sys##name(__MAP(x, __SC_LONG, __VA_ARGS__));    \
+       static inline long __kpatch_do_sys##name(__MAP(x, __SC_DECL, __VA_ARGS__));    \
+       __KPATCH_S390_SYS_STUBx(x, name, __VA_ARGS__);                                 \
+       long __s390x_sys##name(struct pt_regs *regs)                            \
+       {                                                                       \
+               return __se_sys##name(SC_S390_REGS_TO_ARGS(x, __VA_ARGS__));    \
+       }                                                                       \
+       static inline long __se_sys##name(__MAP(x, __SC_LONG, __VA_ARGS__))     \
+       {                                                                       \
+               __MAP(x, __SC_TEST, __VA_ARGS__);                               \
+               return __kpatch_do_sys##name(__MAP(x, __SC_CAST, __VA_ARGS__));        \
+       }                                                                       \
+       static inline long __kpatch_do_sys##name(__MAP(x, __SC_DECL, __VA_ARGS__))
+
+#endif /* KPATCH_SYSCALL_WRAPPERS_V2 */
 
 #elif defined(CONFIG_PPC64)
 
